using System;
using System.Collections;
using Mono.CSharp.Debugger;

namespace Mono.Debugger.Languages.CSharp
{
	internal class CSharpMethod : MethodSource
	{
		int start_row, end_row;
		ArrayList addresses;
		MonoSymbolTableReader reader;
		JitLineNumberEntry[] line_numbers;
		MethodEntry method;
		SourceMethod source_method;
		IMethod imethod;

		public CSharpMethod (MonoSymbolTableReader reader, IMethod imethod,
				     SourceMethod source_method, MethodEntry method,
				     JitLineNumberEntry[] line_numbers)
			: base (imethod, source_method.SourceFile)
		{
			this.reader = reader;
			this.imethod = imethod;
			this.method = method;
			this.source_method = source_method;
			this.line_numbers = line_numbers;
			this.start_row = method.StartRow;
			this.end_row = method.EndRow;
		}

		protected override MethodSourceData ReadSource ()
		{
			ArrayList lines = new ArrayList ();

			for (int i = 0; i < line_numbers.Length; i++) {
				JitLineNumberEntry lne = line_numbers [i];

				lines.Add (new LineEntry (imethod.StartAddress + lne.Address, lne.Line));
			}

			lines.Sort ();
			return new MethodSourceData (start_row, end_row, lines, source_method);
		}

		public override SourceMethod[] MethodLookup (string query)
		{
			string class_name = method.MethodBase.ReflectedType.FullName;
			string full_name = String.Concat (class_name, ".", query);
			return reader.MethodLookup (full_name);
		}
	}
}
